@@ -275,6 +275,8 @@ all_obj_equal <- function(...) {
   is.logical(r)
 }
 
+all_funs <- function(expr) .Call(C_all_funs, expr)
+
 cinv <- function(x) chol2inv(chol(x))
 
 interact_names <- function(l) {
@@ -347,8 +349,8 @@ anyv <- function(x, value) .Call(C_anyallv, x, value, FALSE)
 allv <- function(x, value) .Call(C_anyallv, x, value, TRUE)
 
 
-copyv <- function(X, v, R, ..., invert = FALSE, vind1 = FALSE) {
-  if(is.list(X, ...)) { # Making sure some error is produced if dots are used
+copyv <- function(X, v, R, ..., invert = FALSE, vind1 = FALSE, xlist = FALSE) {
+  if(is.list(X, ...) && !xlist) { # Making sure some error is produced if dots are used
     if(is.list(R)) {
       res <- .mapply(function(x, r) .Call(C_setcopyv, x, v, r, invert, FALSE, vind1),
                      list(unattrib(X), unattrib(R)), NULL)
@@ -359,8 +361,8 @@ copyv <- function(X, v, R, ..., invert = FALSE, vind1 = FALSE) {
   }
   .Call(C_setcopyv, X, v, R, invert, FALSE, vind1)
 }
-setv  <- function(X, v, R, ..., invert = FALSE, vind1 = FALSE) {
-  if(is.list(X, ...)) { # Making sure some error is produced if dots are used
+setv  <- function(X, v, R, ..., invert = FALSE, vind1 = FALSE, xlist = FALSE) {
+  if(is.list(X, ...) && !xlist) { # Making sure some error is produced if dots are used
     if(is.list(R)) {
       .mapply(function(x, r) .Call(C_setcopyv, x, v, r, invert, TRUE, vind1),
               list(unattrib(X), unattrib(R)), NULL)
