@@ -126,90 +126,96 @@ test_that("Using a factor with unused levels does not pose a problem to statisti
 if(requireNamespace("magrittr", quietly = TRUE)) {
 library(magrittr)
 test_that("Testing grouped_df methods", {
-  gdf <- wlddev %>% fsubset(year > 1990, region, income, PCGDP:ODA) %>% fgroup_by(region, income)
-  gdf[["wgt"]] <- round(abs(10*rnorm(fnrow(gdf))), 1)
-  expect_visible(gdf %>% fmean)
-  expect_visible(gdf %>% fmean(wgt))
-  expect_equal(gdf %>% fmean(wgt) %>% slt(-sum.wgt), gdf %>% fmean(wgt, keep.w = FALSE))
-  expect_visible(gdf %>% fmedian)
-  expect_visible(gdf %>% fmedian(wgt))
-  expect_equal(gdf %>% fmedian(wgt) %>% slt(-sum.wgt), gdf %>% fmedian(wgt, keep.w = FALSE))
-  expect_visible(gdf %>% fnth)
-  expect_visible(gdf %>% fnth(0.75))
-  expect_visible(gdf %>% fnth(0.75, wgt))
-  expect_equal(gdf %>% fnth(0.75, wgt) %>% slt(-sum.wgt), gdf %>% fnth(0.75, wgt, keep.w = FALSE))
-  expect_visible(gdf %>% fmode)
-  expect_visible(gdf %>% fmode(wgt))
-  expect_equal(gdf %>% fmode(wgt) %>% slt(-sum.wgt), gdf %>% fmode(wgt, keep.w = FALSE))
-  expect_visible(gdf %>% fsum)
-  expect_visible(gdf %>% fsum(wgt))
-  expect_equal(gdf %>% fsum(wgt) %>% slt(-sum.wgt), gdf %>% fsum(wgt, keep.w = FALSE))
-  expect_visible(gdf %>% fprod)
-  expect_visible(gdf %>% fprod(wgt))
-  expect_equal(gdf %>% fprod(wgt) %>% slt(-prod.wgt), gdf %>% fprod(wgt, keep.w = FALSE))
-  expect_visible(gdf %>% fsd)
-  expect_visible(gdf %>% fsd(wgt))
-  expect_equal(gdf %>% fsd(wgt) %>% slt(-sum.wgt), gdf %>% fsd(wgt, keep.w = FALSE))
-  expect_visible(gdf %>% fvar)
-  expect_visible(gdf %>% fvar(wgt))
-  expect_equal(gdf %>% fvar(wgt) %>% slt(-sum.wgt), gdf %>% fvar(wgt, keep.w = FALSE))
-  expect_visible(gdf %>% fmin)
-  expect_visible(gdf %>% fmax)
-  expect_visible(gdf %>% ffirst)
-  expect_visible(gdf %>% flast)
-  expect_visible(gdf %>% fnobs)
-  expect_visible(gdf %>% fndistinct)
-  expect_visible(gdf %>% collapg)
-  expect_visible(gdf %>% fmean(w = wgt)) # good?
-  expect_equal(gdf %>% collapg(w = wgt) %>% slt(-wgt), gdf %>% collapg(w = wgt, keep.w = FALSE))
-  expect_visible(gdf %>% fscale)
-  expect_visible(gdf %>% fscale(wgt))
-  expect_equal(gdf %>% fscale(wgt) %>% slt(-wgt), gdf %>% fscale(wgt, keep.w = FALSE))
-  expect_visible(gdf %>% STD)
-  expect_visible(gdf %>% STD(wgt))
-  expect_equal(gdf %>% STD(wgt) %>% slt(-wgt), gdf %>% STD(wgt, keep.w = FALSE))
-  expect_equal(gdf %>% fscale, gdf %>% STD(stub = FALSE))
-  expect_visible(gdf %>% fbetween)
-  expect_visible(gdf %>% fbetween(wgt))
-  expect_equal(gdf %>% fbetween(wgt) %>% slt(-wgt), gdf %>% fbetween(wgt, keep.w = FALSE))
-  expect_visible(gdf %>% B)
-  expect_visible(gdf %>% B(wgt))
-  expect_equal(gdf %>% B(wgt) %>% slt(-wgt), gdf %>% B(wgt, keep.w = FALSE))
-  expect_equal(gdf %>% fbetween, gdf %>% B(stub = FALSE))
-  expect_visible(gdf %>% fwithin)
-  expect_visible(gdf %>% fwithin(wgt))
-  expect_equal(gdf %>% fwithin(wgt) %>% slt(-wgt), gdf %>% fwithin(wgt, keep.w = FALSE))
-  expect_visible(gdf %>% W)
-  expect_visible(gdf %>% W(wgt))
-  expect_equal(gdf %>% W(wgt) %>% slt(-wgt), gdf %>% W(wgt, keep.w = FALSE))
-  expect_equal(gdf %>% fwithin, gdf %>% W(stub = FALSE))
-  expect_visible(gdf %>% fcumsum)
-  expect_visible(gdf %>% flag)
-  expect_visible(gdf %>% L)
-  expect_visible(gdf %>% F)
-  expect_true(all_obj_equal(gdf %>% flag, gdf %>% L(stubs = FALSE), gdf %>% F(-1, stubs = FALSE)))
-  expect_true(all_obj_equal(gdf %>% flag(-3:3), gdf %>% L(-3:3), gdf %>% F(3:-3)))
-  expect_visible(gdf %>% fdiff)
-  expect_visible(gdf %>% D)
-  expect_true(all_obj_equal(gdf %>% fdiff, gdf %>% D(stubs = FALSE)))
-  expect_equal(gdf %>% fdiff(-2:2, 1:2), gdf %>% D(-2:2, 1:2))
-  expect_visible(gdf %>% fdiff(rho = 0.95))
-  expect_visible(gdf %>% fdiff(-2:2, 1:2, rho = 0.95))
-  expect_visible(gdf %>% fdiff(log = TRUE))
-  expect_visible(gdf %>% fdiff(-2:2, 1:2, log = TRUE))
-  expect_visible(gdf %>% fdiff(log = TRUE, rho = 0.95))
-  expect_visible(gdf %>% fdiff(-2:2, 1:2, log = TRUE, rho = 0.95))
-  expect_visible(gdf %>% fgrowth)
-  expect_visible(gdf %>% G)
-  expect_true(all_obj_equal(gdf %>% fgrowth, gdf %>% G(stubs = FALSE)))
-  expect_equal(gdf %>% fgrowth(-2:2, 1:2), gdf %>% G(-2:2, 1:2))
-  expect_visible(gdf %>% fgrowth(scale = 1))
-  expect_visible(gdf %>% fgrowth(-2:2, 1:2, scale = 1))
-  expect_visible(gdf %>% fgrowth(logdiff = TRUE))
-  expect_visible(gdf %>% fgrowth(-2:2, 1:2, logdiff = TRUE))
-  expect_visible(gdf %>% fgrowth(logdiff = TRUE, scale = 1))
-  expect_visible(gdf %>% fgrowth(-2:2, 1:2, logdiff = TRUE, scale = 1))
-  expect_equal(BY(gby(iris,Species), sum), BY(nv(gby(iris,Species)), sum))
+  for(sortg in c(TRUE, FALSE)) {
+    for(retgrp in c(TRUE, FALSE)) {
+      gdf <- wlddev %>% fsubset(year > 1990, region, income, PCGDP:ODA) %>% fgroup_by(region, income, return.groups = retgrp, sort = sortg)
+      gdf[["wgt"]] <- round(abs(10*rnorm(fnrow(gdf))), 1)
+      expect_visible(gdf %>% fmean)
+      expect_visible(gdf %>% fmean(wgt))
+      expect_equal(gdf %>% fmean(wgt) %>% slt(-sum.wgt), gdf %>% fmean(wgt, keep.w = FALSE))
+      expect_visible(gdf %>% fmedian)
+      expect_visible(gdf %>% fmedian(wgt))
+      expect_equal(gdf %>% fmedian(wgt) %>% slt(-sum.wgt), gdf %>% fmedian(wgt, keep.w = FALSE))
+      expect_visible(gdf %>% fnth)
+      expect_visible(gdf %>% fnth(0.75))
+      expect_visible(gdf %>% fnth(0.75, wgt))
+      expect_equal(gdf %>% fnth(0.75, wgt) %>% slt(-sum.wgt), gdf %>% fnth(0.75, wgt, keep.w = FALSE))
+      expect_visible(gdf %>% fmode)
+      expect_visible(gdf %>% fmode(wgt))
+      expect_equal(gdf %>% fmode(wgt) %>% slt(-sum.wgt), gdf %>% fmode(wgt, keep.w = FALSE))
+      expect_visible(gdf %>% fsum)
+      expect_visible(gdf %>% fsum(wgt))
+      expect_equal(gdf %>% fsum(wgt) %>% slt(-sum.wgt), gdf %>% fsum(wgt, keep.w = FALSE))
+      expect_visible(gdf %>% fprod)
+      expect_visible(gdf %>% fprod(wgt))
+      expect_equal(gdf %>% fprod(wgt) %>% slt(-prod.wgt), gdf %>% fprod(wgt, keep.w = FALSE))
+      expect_visible(gdf %>% fsd)
+      expect_visible(gdf %>% fsd(wgt))
+      expect_equal(gdf %>% fsd(wgt) %>% slt(-sum.wgt), gdf %>% fsd(wgt, keep.w = FALSE))
+      expect_visible(gdf %>% fvar)
+      expect_visible(gdf %>% fvar(wgt))
+      expect_equal(gdf %>% fvar(wgt) %>% slt(-sum.wgt), gdf %>% fvar(wgt, keep.w = FALSE))
+      expect_visible(gdf %>% fmin)
+      expect_visible(gdf %>% fmax)
+      expect_visible(gdf %>% ffirst)
+      expect_visible(gdf %>% flast)
+      expect_visible(gdf %>% fnobs)
+      expect_visible(gdf %>% fndistinct)
+      expect_visible(gdf %>% collapg)
+      expect_visible(gdf %>% varying)
+      expect_visible(gdf %>% varying(any_group = FALSE))
+      expect_visible(gdf %>% fmean(w = wgt)) # good?
+      expect_equal(gdf %>% collapg(w = wgt) %>% slt(-wgt), gdf %>% collapg(w = wgt, keep.w = FALSE))
+      expect_visible(gdf %>% fscale)
+      expect_visible(gdf %>% fscale(wgt))
+      expect_equal(gdf %>% fscale(wgt) %>% slt(-wgt), gdf %>% fscale(wgt, keep.w = FALSE))
+      expect_visible(gdf %>% STD)
+      expect_visible(gdf %>% STD(wgt))
+      expect_equal(gdf %>% STD(wgt) %>% slt(-wgt), gdf %>% STD(wgt, keep.w = FALSE))
+      expect_equal(gdf %>% fscale, gdf %>% STD(stub = FALSE))
+      expect_visible(gdf %>% fbetween)
+      expect_visible(gdf %>% fbetween(wgt))
+      expect_equal(gdf %>% fbetween(wgt) %>% slt(-wgt), gdf %>% fbetween(wgt, keep.w = FALSE))
+      expect_visible(gdf %>% B)
+      expect_visible(gdf %>% B(wgt))
+      expect_equal(gdf %>% B(wgt) %>% slt(-wgt), gdf %>% B(wgt, keep.w = FALSE))
+      expect_equal(gdf %>% fbetween, gdf %>% B(stub = FALSE))
+      expect_visible(gdf %>% fwithin)
+      expect_visible(gdf %>% fwithin(wgt))
+      expect_equal(gdf %>% fwithin(wgt) %>% slt(-wgt), gdf %>% fwithin(wgt, keep.w = FALSE))
+      expect_visible(gdf %>% W)
+      expect_visible(gdf %>% W(wgt))
+      expect_equal(gdf %>% W(wgt) %>% slt(-wgt), gdf %>% W(wgt, keep.w = FALSE))
+      expect_equal(gdf %>% fwithin, gdf %>% W(stub = FALSE))
+      expect_visible(gdf %>% fcumsum)
+      expect_visible(gdf %>% flag)
+      expect_visible(gdf %>% L)
+      expect_visible(gdf %>% F)
+      expect_true(all_obj_equal(gdf %>% flag, gdf %>% L(stubs = FALSE), gdf %>% F(-1, stubs = FALSE)))
+      expect_true(all_obj_equal(gdf %>% flag(-3:3), gdf %>% L(-3:3), gdf %>% F(3:-3)))
+      expect_visible(gdf %>% fdiff)
+      expect_visible(gdf %>% D)
+      expect_true(all_obj_equal(gdf %>% fdiff, gdf %>% D(stubs = FALSE)))
+      expect_equal(gdf %>% fdiff(-2:2, 1:2), gdf %>% D(-2:2, 1:2))
+      expect_visible(gdf %>% fdiff(rho = 0.95))
+      expect_visible(gdf %>% fdiff(-2:2, 1:2, rho = 0.95))
+      expect_visible(gdf %>% fdiff(log = TRUE))
+      expect_visible(gdf %>% fdiff(-2:2, 1:2, log = TRUE))
+      expect_visible(gdf %>% fdiff(log = TRUE, rho = 0.95))
+      expect_visible(gdf %>% fdiff(-2:2, 1:2, log = TRUE, rho = 0.95))
+      expect_visible(gdf %>% fgrowth)
+      expect_visible(gdf %>% G)
+      expect_true(all_obj_equal(gdf %>% fgrowth, gdf %>% G(stubs = FALSE)))
+      expect_equal(gdf %>% fgrowth(-2:2, 1:2), gdf %>% G(-2:2, 1:2))
+      expect_visible(gdf %>% fgrowth(scale = 1))
+      expect_visible(gdf %>% fgrowth(-2:2, 1:2, scale = 1))
+      expect_visible(gdf %>% fgrowth(logdiff = TRUE))
+      expect_visible(gdf %>% fgrowth(-2:2, 1:2, logdiff = TRUE))
+      expect_visible(gdf %>% fgrowth(logdiff = TRUE, scale = 1))
+      expect_visible(gdf %>% fgrowth(-2:2, 1:2, logdiff = TRUE, scale = 1))
+      expect_equal(BY(gby(iris,Species), sum), BY(nv(gby(iris,Species)), sum))
+    }
+  }
 })
 }
 
